@@ -1,6 +1,6 @@
 import math
 import torch
-from torch import BoolTensor, LongTensor, Tensor
+from torch import BoolTensor, LongTensor, FloatTensor, Tensor
 import torch.nn.functional as F
 from collections import deque
 
@@ -27,6 +27,7 @@ __all__ = [
     "bind_sequence",
     "ngrams",
     "hash_table",
+    "graph",
     "map_range",
     "value_to_index",
     "index_to_value",
@@ -41,7 +42,7 @@ def identity_hv(
     device=None,
     requires_grad=False,
 ) -> Tensor:
-    """Creates a set of identity hypervector.
+    """Creates a set of identity hypervectors.
 
     When bound with a random-hypervector :math:`x`, the result is :math:`x`.
 
@@ -174,24 +175,20 @@ def random_hv(
     if dtype == torch.uint8:
         raise ValueError("Unsigned integer hypervectors are not supported.")
 
+    size = (num_embeddings, embedding_dim)
     if dtype in {torch.complex64, torch.complex128}:
         dtype = torch.float if dtype == torch.complex64 else torch.double
 
-        angle = torch.empty(num_embeddings, embedding_dim, dtype=dtype, device=device)
+        angle = torch.empty(size, dtype=dtype, device=device)
         angle.uniform_(-math.pi, math.pi)
-        magnitude = torch.ones(
-            num_embeddings, embedding_dim, dtype=dtype, device=device
-        )
+        magnitude = torch.ones(size, dtype=dtype, device=device)
 
         result = torch.polar(magnitude, angle)
         result.requires_grad = requires_grad
         return result
 
     select = torch.empty(
-        (
-            num_embeddings,
-            embedding_dim,
-        ),
+        size,
         dtype=torch.bool,
     ).bernoulli_(1.0 - sparsity, generator=generator)
 
@@ -485,12 +482,13 @@ def bind(input: Tensor, other: Tensor) -> Tensor:
 
     Examples::
 
-        >>> x = functional.random_hv(2, 3)
-        >>> x
-        tensor([[ 1., -1., -1.],
-                [ 1.,  1.,  1.]])
-        >>> functional.bind(x[0], x[1])
-        tensor([ 1., -1., -1.])
+        >>> a, b = functional.random_hv(2, 10)
+        >>> a
+        tensor([-1.,  1., -1., -1.,  1.,  1., -1.,  1.,  1., -1.])
+        >>> b
+        tensor([-1., -1.,  1.,  1., -1., -1.,  1.,  1.,  1.,  1.])
+        >>> functional.bind(a, b)
+        tensor([ 1., -1., -1., -1., -1., -1., -1.,  1.,  1., -1.])
 
     """
     dtype = input.dtype
@@ -574,12 +572,13 @@ def bundle(input: Tensor, other: Tensor, *, tie: BoolTensor = None) -> Tensor:
 
     Examples::
 
-        >>> x = functional.random_hv(2, 3)
-        >>> x
-        tensor([[ 1.,  1.,  1.],
-                [-1.,  1., -1.]])
-        >>> functional.bundle(x[0], x[1])
-        tensor([0., 2., 0.])
+        >>> a, b = functional.random_hv(2, 10)
+        >>> a
+        tensor([ 1., -1.,  1., -1., -1.,  1., -1., -1.,  1., -1.])
+        >>> b
+        tensor([ 1., -1., -1.,  1.,  1.,  1., -1.,  1., -1.,  1.])
+        >>> functional.bundle(a, b)
+        tensor([ 2., -2.,  0.,  0.,  0.,  2., -2.,  0.,  0.,  0.])
 
     """
     dtype = input.dtype
@@ -895,6 +894,93 @@ def multiset(input: Tensor) -> Tensor:
     return torch.sum(input, dim=dim, dtype=dtype)
 
 
+def randsel(
+    input: Tensor, other: Tensor, *, p: float = 0.5, generator: torch.Generator = None
+) -> Tensor:
+    r"""Bundles two hypervectors by selecting random elements.
+
+    A bundling operation is used to aggregate information into a single hypervector.
+    The resulting hypervector has elements selected at random from input or other.
+
+    .. math::
+
+        \oplus: \mathcal{H} \times \mathcal{H} \to \mathcal{H}
+
+    Aliased as ``torchhd.randsel``.
+
+    Args:
+        input (Tensor): input hypervector
+        other (Tensor): other input hypervector
+        p (float, optional): probability of selecting elements from the input hypervector. Default: 0.5.
+        generator (``torch.Generator``, optional): a pseudorandom number generator for sampling.
+
+    Shapes:
+        - Input: :math:`(*)`
+        - Other: :math:`(*)`
+        - Output: :math:`(*)`
+
+    Examples::
+
+        >>> a, b = functional.random_hv(2, 10)
+        >>> a
+        tensor([ 1., -1.,  1., -1.,  1., -1., -1., -1., -1.,  1.])
+        >>> b
+        tensor([ 1., -1.,  1., -1.,  1.,  1., -1.,  1., -1.,  1.])
+        >>> functional.randsel(a, b)
+        tensor([ 1., -1.,  1., -1.,  1.,  1., -1.,  1., -1.,  1.])
+
+    """
+    select = torch.empty_like(input, dtype=torch.bool)
+    select.bernoulli_(1 - p, generator=generator)
+    return input.where(select, other)
+
+
+def multirandsel(
+    input: Tensor, *, p: FloatTensor = None, generator: torch.Generator = None
+) -> Tensor:
+    r"""Bundling multiple hypervectors by sampling random elements.
+
+    Bundles all the input hypervectors together.
+    The resulting hypervector has elements selected at random from the input tensor of hypervectors.
+
+    .. math::
+
+        \bigoplus_{i=0}^{n-1} V_i
+
+    Args:
+        input (Tensor): input hypervector tensor
+        p (FloatTensor, optional): probability of selecting elements from the input hypervector. Default: uniform.
+        generator (``torch.Generator``, optional): a pseudorandom number generator for sampling.
+
+    Shapes:
+        - Input: :math:`(*, n, d)`
+        - Probability (p): :math:`(*, n)`
+        - Output: :math:`(*, d)`
+
+    Examples::
+
+        >>> x = functional.random_hv(5, 10)
+        >>> x
+        tensor([[-1.,  1.,  1.,  1., -1.,  1.,  1., -1.,  1., -1.],
+                [-1.,  1.,  1.,  1., -1.,  1., -1., -1., -1., -1.],
+                [-1., -1.,  1., -1., -1.,  1., -1.,  1.,  1., -1.],
+                [ 1.,  1., -1.,  1., -1., -1.,  1., -1.,  1.,  1.],
+                [ 1., -1., -1.,  1.,  1.,  1.,  1., -1., -1., -1.]])
+        >>> functional.multirandsel(x)
+        tensor([ 1., -1., -1.,  1., -1.,  1.,  1.,  1.,  1., -1.])
+
+    """
+    d = input.size(-1)
+    device = input.device
+
+    if p is None:
+        p = torch.ones(input.shape[:-1], dtype=torch.float, device=device)
+
+    select = torch.multinomial(p, d, replacement=True, generator=generator)
+    select.unsqueeze_(-2)
+    return input.gather(-2, select).squeeze(-2)
+
+
 multibundle = multiset
 
 
@@ -920,13 +1006,15 @@ def multibind(input: Tensor) -> Tensor:
 
     Examples::
 
-        >>> x = functional.random_hv(3, 3)
+        >>> x = functional.random_hv(5, 10)
         >>> x
-        tensor([[ 1.,  1.,  1.],
-                [-1.,  1.,  1.],
-                [-1.,  1., -1.]])
+        tensor([[ 1., -1., -1., -1., -1.,  1.,  1.,  1., -1.,  1.],
+                [ 1.,  1., -1., -1.,  1.,  1., -1., -1.,  1., -1.],
+                [-1., -1.,  1., -1., -1., -1., -1.,  1., -1., -1.],
+                [-1.,  1., -1.,  1.,  1., -1., -1., -1.,  1.,  1.],
+                [-1., -1.,  1., -1.,  1., -1.,  1.,  1., -1.,  1.]])
         >>> functional.multibind(x)
-        tensor([ 1.,  1., -1.])
+        tensor([-1., -1., -1.,  1.,  1., -1., -1.,  1., -1.,  1.])
 
     """
     dtype = input.dtype
@@ -1031,7 +1119,7 @@ def hash_table(keys: Tensor, values: Tensor) -> Tensor:
 
     .. math::
 
-        \bigoplus_{i = 0}^{m - 1} K_i \otimes V_i
+        \bigoplus_{i = 0}^{n - 1} K_i \otimes V_i
 
     Args:
         keys (Tensor): The keys hypervectors, must be the same shape as values.
@@ -1066,7 +1154,7 @@ def bundle_sequence(input: Tensor) -> Tensor:
 
     .. math::
 
-        \bigoplus_{i=0}^{m-1} \Pi^{m - i - 1}(V_i)
+        \bigoplus_{i=0}^{n-1} \Pi^{n - i - 1}(V_i)
 
     Args:
         input (Tensor): The hypervector values.
@@ -1105,7 +1193,7 @@ def bind_sequence(input: Tensor) -> Tensor:
 
     .. math::
 
-        \bigotimes_{i=0}^{m-1} \Pi^{m - i - 1}(V_i)
+        \bigotimes_{i=0}^{n-1} \Pi^{n - i - 1}(V_i)
 
     Args:
         input (Tensor): The hypervector values.
@@ -1141,6 +1229,48 @@ def bind_sequence(input: Tensor) -> Tensor:
     return multibind(permuted)
 
 
+def graph(input: Tensor, *, directed=False) -> Tensor:
+    r"""Graph from node hypervector pairs.
+
+    If ``directed=False`` this computes:
+
+    .. math::
+
+        \bigoplus_{i = 0}^{n - 1} V_{0,i} \otimes V_{1,i}
+
+    If ``directed=True`` this computes:
+
+    .. math::
+
+        \bigoplus_{i = 0}^{n - 1} V_{0,i} \otimes \Pi(V_{1,i})
+
+    Args:
+        input (Tensor): tensor containing pairs of node hypervectors that share an edge.
+        directed (bool, optional): specify if the graph is directed or not. Default: ``False``.
+
+    Shapes:
+        - Input: :math:`(*, 2, n, d)`
+        - Output: :math:`(*, d)`
+
+    Examples::
+            >>> edges = torch.tensor([[0, 0, 1, 2], [1, 2, 2, 3]])
+            >>> node_embedding = embeddings.Random(4, 10000)
+            >>> edges_hv = node_embedding(edges)
+            >>> graph = functional.graph(edges_hv)
+            >>> neighbors = unbind(graph, node_embedding.weight[0])
+            >>> cosine_similarity(neighbors, node_embedding.weight)
+            tensor([0.0006, 0.5017, 0.4997, 0.0048])
+
+    """
+    to_nodes = input[..., 0, :, :]
+    from_nodes = input[..., 1, :, :]
+
+    if directed:
+        from_nodes = permute(from_nodes)
+
+    return multiset(bind(to_nodes, from_nodes))
+
+
 def map_range(
     input: Tensor,
     in_min: float,
